# Copyright (C) 2008 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the LGPL (>=v2.1)
# Read the file COPYING coming with the software for details.

import unittest
from imapsupport import ImapConnection, ImapError, default_config


class AnnotationTests(unittest.TestCase):

    def setUp(self):
        self.config = default_config()
        self.imap = ImapConnection(self.config.host, self.config.port,
                                   self.config.login, self.config.password,
                                   self.config.ssl)

    def tearDown(self):
        self.imap.logout()

    def ensure_mailbox_unknown(self, mailbox):
        try:
            self.imap.delete(mailbox)
        except ImapError:
            pass

    def ensure_mailbox_new_and_empty(self, mailbox):
        self.ensure_mailbox_unknown(mailbox)
        self.imap.create(mailbox)

    def check_bad_parameters(self, command, args, message_template):
        try:
            getattr(self.imap, command.lower())(*args)
        except self.imap.error_class, e:
            self.failUnless(str(e).startswith("%s command error: BAD"
                                              % command.upper()))
        else:
            self.fail(message_template % command.upper())

    def check_imap_error(self, command, args, message_template,
                         response_type="NO", response=None):
        try:
            getattr(self.imap, command.lower())(*args)
        except ImapError, e:
            self.failUnless(e.imap_result[0] == response_type)
            if response:
                self.failUnless(response in e.imap_result[1])
        else:
            self.fail(message_template % command.upper())


class TestCapabilities(AnnotationTests):

    def test_capabilities(self):
        self.failUnless("ANNOTATEMORE" in self.imap.capabilities)


class TestGetAnnotation(AnnotationTests):

    def setUp(self):
        AnnotationTests.setUp(self)
        self.mailbox = "INBOX"
        self.entry = "/vendor/kolab/folder-type"
        self.attribute = "value.shared"

    def test_getannotation_no_parameters(self):
        self.check_bad_parameters("GETANNOTATION", (),
                                  "%s without parameters did not return an"
                                  " error")

    def test_getannotation_one_parameters(self):
        self.check_bad_parameters("GETANNOTATION", ("INBOX",),
                                  "%s with only one parameter did not return an"
                                  " error")

    def test_getannotation_two_parameters(self):
        self.check_bad_parameters("GETANNOTATION",
                                  ("INBOX", "/vendor/kolab/test"),
                                  "%s with two parameters did not return an"
                                  " error")

    def test_getannotation_entry_no_slash(self):
        self.check_bad_parameters("GETANNOTATION",
                                  ("INBOX", "comment", "value.shared"),
                                  "%s with entry not starting with slash"
                                  " did not return an error")

    def test_getannotation_attribute_nil(self):
        self.check_bad_parameters("GETANNOTATION",
                                  (self.mailbox, self.entry, None),
                                  "%s with attribute name NIL did not return"
                                  " an error")

    def test_getannotation_nonexisting_mailbox(self):
        mailbox = "INBOX/nowhere"
        # make sure mailbox doesn't exist
        self.ensure_mailbox_unknown(mailbox)
        self.check_imap_error("GETANNOTATION",
                              (mailbox, "/comment", "value.shared"),
                              "%s for non-existing mailbox did not"
                              " return an error")

    def test_getannotation_unknown_namespace(self):
        mailbox = "unknown/foo"
        # make sure mailbox doesn't exist
        self.ensure_mailbox_unknown(mailbox)
        self.check_imap_error("GETANNOTATION",
                              (mailbox, "/comment", ("value.shared", "foo")),
                              "%s for unknown mailbox namespace did not"
                              " return an error")

    def test_getannotation(self):
        annotation = self.imap.getannotation("INBOX", "/vendor/kolab/test",
                                             "value.shared")
        self.assertEquals(annotation, [])

    def test_getannotation_lists_wuth_single_elements(self):
        mailbox = "INBOX/annotated"
        entry = "/vendor/kolab/test"
        self.ensure_mailbox_new_and_empty(mailbox)
        self.imap.setannotation(mailbox, entry,
                                ["value.shared", "an annotation"])
        annotation = self.imap.getannotation(mailbox, [entry], ["value.shared"])
        self.assertEquals(annotation,
                          ['"INBOX/annotated" "/vendor/kolab/test"'
                           ' ("value.shared" "an annotation")'])


class TestSetAnnotation(AnnotationTests):

    def setUp(self):
        AnnotationTests.setUp(self)
        self.mailbox = "INBOX/Calendar"
        self.entry = "/vendor/kolab/folder-type"
        self.attribute = "value.shared"
        self.value = "event"
        self.ensure_mailbox_new_and_empty(self.mailbox)

    def test_setannotation_no_argument(self):
        self.check_bad_parameters("SETANNOTATION", (),
                                  "%s with no parameters did not return an"
                                  " error")

    def test_setannotation_one_argument(self):
        self.check_bad_parameters("SETANNOTATION", (self.mailbox),
                                  "%s with one parameter did not return an"
                                  " error")

    def test_setannotation_two_argument(self):
        self.check_bad_parameters("SETANNOTATION", (self.mailbox, self.entry),
                                  "%s with two parameters did not return an"
                                  " error")

    def test_setannotation_entry_no_slash(self):
        self.check_bad_parameters("SETANNOTATION",
                                  (self.mailbox, "comment",
                                   [self.attribute, self.value]),
                                  "%s with entry not starting with slash"
                                  " did not return an error")

    def test_setannotation_attribute_no_list(self):
        self.check_bad_parameters("SETANNOTATION",
                                  (self.mailbox, self.entry, self.attribute),
                                  "%s with two parameters did not return an"
                                  " error")

    def test_setannotation_attribute_odd_list(self):
        self.check_bad_parameters("SETANNOTATION",
                                  (self.mailbox, self.entry,
                                   ["folder-type", "mail", "entry"]),
                                  "%s with two parameters did not return an"
                                  " error")

    def test_setannotation_attribute_empty_list(self):
        self.check_bad_parameters("SETANNOTATION",
                                  (self.mailbox, self.entry, []),
                                  "%s with empty attribute list did not return"
                                  " an error")

    def test_setannotation_attribute_nil(self):
        self.check_bad_parameters("SETANNOTATION",
                                  (self.mailbox, self.entry, [None, "event"]),
                                  "%s with attribute name NIL did not return"
                                  " an error")

    def test_setannotation_attribute_without_period(self):
        self.check_bad_parameters("SETANNOTATION",
                                  (self.mailbox, self.entry,
                                   ["folder-type", "mail"]),
                                  "%s with attribute without period did not"
                                  " return an error")

    def test_setannotation_attribute_not_ending_in_priv_or_shared(self):
        self.check_bad_parameters("SETANNOTATION",
                                  (self.mailbox, self.entry,
                                   ["value.type", "mail"]),
                                  "%s with attribute not ending in priv or"
                                  " shared did not return an error")

    def test_setannotation_nonexisting_mailbox(self):
        mailbox = "INBOX/nowhere"
        self.ensure_mailbox_unknown(mailbox)
        self.check_imap_error("SETANNOTATION",
                              (mailbox, "/comment", ("value.shared", "foo")),
                              "%s for non-existing mailbox did not"
                              " return an error")

    def test_setannotation_unknown_namespace(self):
        mailbox = "unknown/foo"
        self.ensure_mailbox_unknown(mailbox)
        self.check_imap_error("SETANNOTATION",
                              (mailbox, "/comment", ("value.shared", "foo")),
                              "%s for unknown mailbox namespace did not"
                              " return an error")

    def test_setannotation_public(self):
        self.imap.setannotation(self.mailbox, self.entry,
                                [self.attribute, self.value])
        annotation = self.imap.getannotation(self.mailbox, self.entry,
                                             self.attribute)
        self.assertEquals(annotation,
                          ['"%s" "%s" ("%s" "%s")'
                           % (self.mailbox, self.entry,
                              self.attribute, self.value)])

    def test_setannotation_deleting_public(self):
        self.imap.setannotation(self.mailbox, self.entry,
                                [self.attribute, self.value])
        self.imap.setannotation(self.mailbox, self.entry,
                                [self.attribute, None])
        self.assertEquals(self.imap.getannotation(self.mailbox, self.entry,
                                                  self.attribute),
                          [])


class TestMailboxChanges(AnnotationTests):

    def test_mailbox_renaming(self):
        self.ensure_mailbox_unknown("INBOX/old")
        self.ensure_mailbox_unknown("INBOX/new")

        self.imap.create("INBOX/old")
        self.imap.setannotation("INBOX/old", "/vendor/kolab/folder-test",
                                ["value.shared", "some value"])
        self.imap.rename("INBOX/old", "INBOX/new")

        self.assertEquals(self.imap.getannotation("INBOX/new",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          ['"INBOX/new" "/vendor/kolab/folder-test"'
                           ' ("value.shared" "some value")'])

    def test_mailbox_renaming_with_inferior_mailboxes(self):
        self.ensure_mailbox_new_and_empty("INBOX/this")
        self.ensure_mailbox_new_and_empty("INBOX/this/child")
        self.ensure_mailbox_new_and_empty("INBOX/this/child/grandchild")
        self.ensure_mailbox_unknown("INBOX/that")
        self.ensure_mailbox_unknown("INBOX/that/child")
        self.ensure_mailbox_unknown("INBOX/that/child/grandchild")

        self.imap.setannotation("INBOX/this/child",
                                "/vendor/kolab/folder-test",
                                ["value.shared", "child value"])
        self.imap.setannotation("INBOX/this/child/grandchild",
                                "/vendor/kolab/folder-test",
                                ["value.shared", "grand child value"])
        self.imap.rename("INBOX/this", "INBOX/that")

        self.assertEquals(self.imap.getannotation("INBOX/that/child",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          ['"INBOX/that/child" "/vendor/kolab/folder-test"'
                           ' ("value.shared" "child value")'])
        self.assertEquals(self.imap.getannotation("INBOX/that/child/grandchild",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          ['"INBOX/that/child/grandchild"'
                           ' "/vendor/kolab/folder-test"'
                           ' ("value.shared" "grand child value")'])

    def test_mailbox_renaming_with_common_prefixes(self):
        self.ensure_mailbox_new_and_empty("INBOX/this")
        self.ensure_mailbox_new_and_empty("INBOX/this/child")
        self.ensure_mailbox_new_and_empty("INBOX/thisother")
        self.ensure_mailbox_new_and_empty("INBOX/thisother/child")
        self.ensure_mailbox_unknown("INBOX/that")
        self.ensure_mailbox_unknown("INBOX/that/child")

        self.imap.setannotation("INBOX/this/child",
                                "/vendor/kolab/folder-test",
                                ["value.shared", "this child value"])
        self.imap.setannotation("INBOX/thisother/child",
                                "/vendor/kolab/folder-test",
                                ["value.shared", "other child value"])
        self.imap.rename("INBOX/this", "INBOX/that")

        self.assertEquals(self.imap.getannotation("INBOX/that/child",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          ['"INBOX/that/child" "/vendor/kolab/folder-test"'
                           ' ("value.shared" "this child value")'])
        self.assertEquals(self.imap.getannotation("INBOX/thisother/child",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          ['"INBOX/thisother/child"'
                           ' "/vendor/kolab/folder-test"'
                           ' ("value.shared" "other child value")'])

    def test_mailbox_deletion(self):
        self.ensure_mailbox_new_and_empty("INBOX/deletiontest")
        self.imap.setannotation("INBOX/deletiontest",
                                "/vendor/kolab/folder-test",
                                ["value.shared", "some value"])
        self.assertEquals(self.imap.getannotation("INBOX/deletiontest",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          ['"INBOX/deletiontest" "/vendor/kolab/folder-test"'
                           ' ("value.shared" "some value")'])
        self.imap.delete("INBOX/deletiontest")
        self.imap.create("INBOX/deletiontest")
        self.assertEquals(self.imap.getannotation("INBOX/deletiontest",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          [])

    def test_mailbox_deletion_with_common_prefixes(self):
        self.ensure_mailbox_new_and_empty("INBOX/deletiontest")
        self.ensure_mailbox_new_and_empty("INBOX/deletiontestother")

        self.imap.setannotation("INBOX/deletiontest",
                                "/vendor/kolab/folder-test",
                                ["value.shared", "some value"])
        self.imap.setannotation("INBOX/deletiontestother",
                                "/vendor/kolab/folder-test",
                                ["value.shared", "other value"])
        self.imap.delete("INBOX/deletiontest")
        self.imap.create("INBOX/deletiontest")
        self.assertEquals(self.imap.getannotation("INBOX/deletiontest",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          [])
        self.assertEquals(self.imap.getannotation("INBOX/deletiontestother",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          ['"INBOX/deletiontestother"'
                           ' "/vendor/kolab/folder-test"'
                           ' ("value.shared" "other value")'])

    def test_mailbox_deletion_with_inferior_mailboxes(self):
        self.ensure_mailbox_new_and_empty("INBOX/deletion")
        self.ensure_mailbox_new_and_empty("INBOX/deletion/child")

        self.imap.setannotation("INBOX/deletion/child",
                                "/vendor/kolab/folder-test",
                                ["value.shared", "child value"])
        self.imap.delete("INBOX/deletion")
        self.assertEquals(self.imap.getannotation("INBOX/deletion/child",
                                                  "/vendor/kolab/folder-test",
                                                  "value.shared"),
                          ['"INBOX/deletion/child" "/vendor/kolab/folder-test"'
                           ' ("value.shared" "child value")'])

    def test_renaming_and_deletion_with_large_datasets(self):
        self.ensure_mailbox_new_and_empty("INBOX/many-annotations")
        self.ensure_mailbox_unknown("INBOX/many-renamed")

        value = "x" * 1024
        for i in range(1024):
            entry = "/public/vendor/kolab/numbered_%03d" % i
            self.imap.setannotation("INBOX/many-annotations",
                                    entry, ["value.shared", value])

        self.imap.rename("INBOX/many-annotations", "INBOX/many-renamed")

        for i in range(1024):
            entry = "/public/vendor/kolab/numbered_%03d" % i
            self.assertEquals(self.imap.getannotation("INBOX/many-renamed",
                                                      entry, "value.shared"),
                              ['"INBOX/many-renamed" "%s" ("value.shared" "%s")'
                               % (entry, value)])

        self.imap.delete("INBOX/many-renamed")



if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2008 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the LGPL (>=v2.1)
# Read the file COPYING coming with the software for details.

import os
import imaplib
import ConfigParser

def convert_bool(s):
    s = s.lower()
    if s in ("true", "yes", "1"):
        return True
    if s in ("false", "no", "0"):
        return False
    raise ValueError("cannot determine boolean value of %r" % (s,))


class ImapConfig(object):

    def __init__(self, configfile):
        parser = ConfigParser.SafeConfigParser()
        parser.read([configfile])
        for name, conv in [("host", str), ("port", int),
                           ("login", str), ("password", str),
                           ("ssl", convert_bool)]:
            setattr(self, name, conv(parser.get("imap", name)))


default_config_file = os.path.join(os.path.dirname(__file__), "imaptest.conf")

def default_config():
    return ImapConfig(default_config_file)


class AnnotationsMixin:

    def install_commands_in_imaplib(self):
        imaplib.Commands['GETANNOTATION'] = ('AUTH', 'SELECTED')
        imaplib.Commands['SETANNOTATION'] = ('AUTH', 'SELECTED')

    def to_argument_string(self, arg):
        if isinstance(arg, basestring):
            return self._quote(arg)
        elif arg is None:
            return "NIL"
        else:
            return "(%s)" % (" ".join(self.to_argument_string(item)
                                      for item in arg),)

    def getannotation(self, *args):
        # normally we would implement this command like the other
        # commands in IMAP with an explicit list of parameters.  For
        # tests it's useful to be able to omit parameters or to send too
        # many, so we allow an arbitrary number of parameters
        self.install_commands_in_imaplib()
        args = tuple(self.to_argument_string(item) for item in args)
        typ, dat = self._simple_command('GETANNOTATION', *args)
        return self._untagged_response(typ, dat, 'ANNOTATION')

    def setannotation(self, *args):
        # normally we would implement this command like the other
        # commands in IMAP with an explicit list of parameters.  For
        # tests it's useful to be able to omit parameters or to send too
        # many, so we allow an arbitrary number of parameters
        self.install_commands_in_imaplib()
        args = tuple(self.to_argument_string(item) for item in args)
        return self._simple_command('SETANNOTATION', *args)


class IMAP4_annotations(imaplib.IMAP4, AnnotationsMixin):

    pass


class IMAP4_SSL_annotations(imaplib.IMAP4_SSL, AnnotationsMixin):

    pass



class ImapError(Exception):

    def __init__(self, host, port, imap_result):
        Exception.__init__(self, "Unexpected response from imap server"
                           " (%r:%d):%r" % (host, port, imap_result))
        self.imap_result = imap_result


class ImapConnection(object):

    def __init__(self, host, port, login, password, use_ssl):
        self.host = host
        self.port = port
        if use_ssl:
            cls = IMAP4_SSL_annotations
        else:
            cls = IMAP4_annotations
        self.error_class = cls.error
        self.imap = cls(host, port)
        self.auth_plain(login, password)

    def check_res(self, res, ok_responses=("OK",)):
        assert len(res) == 2
        if res[0] not in ok_responses:
            raise ImapError(self.host, self.port, res)
        data = res[1]
        # for some reason imaplib uses a one-element list containing
        # None to indicate that no untagged responses came from the
        # server.  Translate this to an empty list.
        if data == [None]:
            data = []
        return data

    def auth_plain(self, login, password):
        self.check_res(self.imap.login(login, password))

    def logout(self):
        self.check_res(self.imap.logout(), ok_responses=("OK", "BYE"))

    @property
    def capabilities(self):
        return self.imap.capabilities

    def create(self, mailbox):
        self.check_res(self.imap.create(mailbox))

    def delete(self, mailbox):
        self.check_res(self.imap.delete(mailbox))

    def rename(self, oldmailbox, newmailbox):
        self.check_res(self.imap.rename(oldmailbox, newmailbox))

    def getannotation(self, *args):
        return self.check_res(self.imap.getannotation(*args))

    def setannotation(self, *args):
        return self.check_res(self.imap.setannotation(*args))



#! /usr/bin/env python
# Copyright (C) 2004, 2005, 2006, 2007, 2008 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the LGPL (>=v2.1)
# Read the file COPYING coming with the software for details.

"""
Main entry point for the test suite.

Just run this file as a python script to execute all tests
"""

import os
import sys
import unittest
import optparse

test_dir = os.path.dirname(__file__)
sys.path.append(os.path.join(test_dir, os.pardir))

def find_test_modules(dirname, package = None):
    """Return a list the names of the test modules in the directory dirname

    The return value is a list of names that can be passed to
    unittest.defaultTestLoader.loadTestsFromNames.  Each name of the
    list is the name of a pure python module, one for each file in
    dirname whose name starts with 'test' and ends with '.py'.

    The optional parameter package should be the name of the python
    package whose directory is dirname.  If package is given all names
    in the returned list will be prefixed with package and a dot.
    """
    if package:
        prefix = package + "."
    else:
        prefix = ""

    suffix = ".py"
    return [prefix + name[:-len(suffix)]
            for name in os.listdir(dirname)
                if name.startswith("test") and name.endswith(suffix)]


def main():
    """Run all the tests in the test suite"""

    parser = optparse.OptionParser()
    parser.set_defaults(verbosity=1, test_private_metadata=False)
    parser.add_option("-v", "--verbose", action="store_const", const=2,
                      dest="verbosity")
    parser.add_option("--test-private-metadata", action="store_const",
                      const=True)
    parser.add_option("--config")
    opts, rest = parser.parse_args()

    if opts.config is not None:
        import imapsupport
        imapsupport.default_config_file = opts.config

    # Build the list of test names.  If names were given on the command
    # line, run exactly those.  Othwerwise build a default list of
    # names.
    if rest:
        names = rest
    else:
        names = find_test_modules(test_dir)
        if not opts.test_private_metadata:
            names = [name for name in names if not name.endswith("_private")]

    suite = unittest.defaultTestLoader.loadTestsFromNames(names)
    runner = unittest.TextTestRunner(verbosity=opts.verbosity)
    result = runner.run(suite)
    sys.exit(not result.wasSuccessful())


if __name__ == "__main__":
    main()



# Copyright (C) 2008 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the LGPL (>=v2.1)
# Read the file COPYING coming with the software for details.

import unittest
from imapsupport import ImapConnection, ImapError, default_config

from test_annotatemore import AnnotationTests

class TestSetAnnotation(AnnotationTests):

    def setUp(self):
        AnnotationTests.setUp(self)
        self.mailbox = "INBOX/Calendar"
        self.entry = "/vendor/kolab/folder-type"
        self.attribute = "value.shared"
        self.value = "event"
        self.ensure_mailbox_new_and_empty(self.mailbox)

    def test_setannotation_private(self):
        attribute = "value.priv"
        value = "private value"
        self.imap.setannotation(self.mailbox, self.entry, [attribute, value])
        annotation = self.imap.getannotation(self.mailbox, self.entry,
                                             attribute)
        self.assertEquals(annotation,
                          ['"%s" "%s" ("%s" "%s")'
                           % (self.mailbox, self.entry, attribute, value)])

    def test_setannotation_deleting_private(self):
        attribute = "value.priv"
        value = "private value"
        self.imap.setannotation(self.mailbox, self.entry, [attribute, value])
        self.imap.setannotation(self.mailbox, self.entry, [attribute, None])
        annotation = self.imap.getannotation(self.mailbox, self.entry,
                                             attribute)
        self.assertEquals(annotation, [])


class TestMisc(AnnotationTests):

    def setUp(self):
        AnnotationTests.setUp(self)
        self.mailbox = "INBOX/Tasks"
        self.entry = "/vendor/kolab/folder-test"
        self.attribute = "value"
        self.attribute_shared = self.attribute + ".shared"
        self.attribute_private = self.attribute + ".priv"
        self.value = "task"
        self.ensure_mailbox_new_and_empty(self.mailbox)

    def test_public_and_private_are_different_namespaces(self):
        value_shared = "shared value"
        value_private = "private value"
        self.imap.setannotation(self.mailbox, self.entry,
                                [self.attribute_shared, value_shared])
        self.imap.setannotation(self.mailbox, self.entry,
                                [self.attribute_private, value_private])
        self.assertEquals(self.imap.getannotation(self.mailbox, self.entry,
                                                  self.attribute_shared),
                          ['"%s" "%s" ("%s" "%s")'
                           % (self.mailbox, self.entry,
                              self.attribute_shared, value_shared)])
        self.assertEquals(self.imap.getannotation(self.mailbox, self.entry,
                                                  self.attribute_private),
                          ['"%s" "%s" ("%s" "%s")'
                           % (self.mailbox, self.entry,
                              self.attribute_private, value_private)])

    def test_getannotation_attribute_without_suffix(self):
        value_shared = "shared value"
        value_private = "private value"
        self.imap.setannotation(self.mailbox, self.entry,
                                [self.attribute_shared, value_shared])
        self.imap.setannotation(self.mailbox, self.entry,
                                [self.attribute_private, value_private])
        self.assertEquals(sorted(self.imap.getannotation(self.mailbox,
                                                         self.entry,
                                                         self.attribute)),
                          sorted(['"%s" "%s" ("%s" "%s")'
                                  % (self.mailbox, self.entry,
                                     self.attribute_shared, value_shared),
                                  '"%s" "%s" ("%s" "%s")'
                                  % (self.mailbox, self.entry,
                                     self.attribute_private, value_private),]))




if __name__ == "__main__":
    unittest.main()
